// Copyright (C) 2023 OWO Games - All Rights Reserved
#include <OWOImport/Public/ImportAuthFactory.h>
#include "Containers/UnrealString.h"
#include "Misc/FileHelper.h"
#include "Templates/SharedPointer.h"
#include "EditorFramework/AssetImportData.h"


UImportAuthFactory::UImportAuthFactory(const FObjectInitializer& ObjectInitializer) : Super(ObjectInitializer)
{
    Formats.Add(FString(TEXT("authowo;")) + NSLOCTEXT("UImportAuthFactory", "AuthFormat", "Auth file").ToString());
    SupportedClass = UImportedAuth::StaticClass();
    bCreateNew = false;
    bEditorImport = true;
    Formats.Add(TEXT("authowo; OWO Auth"));
}

UObject* UImportAuthFactory::FactoryCreateFile(UClass* InClass, UObject* InParent, FName InName, EObjectFlags Flags, const FString& Filename, const TCHAR* Parms, FFeedbackContext* Warn, bool& bOutOperationCanceled)
{
    FString result;

    FFileHelper::LoadFileToString(result, *Filename);

    std::string code = std::string(TCHAR_TO_UTF8(*result));

    UImportedAuth* OWOSensation = NewObject<UImportedAuth>(InParent, UImportedAuth::StaticClass(), InName, Flags);

    OWOSensation->SensationsData = code.c_str();

    OWOSensation->AssetImportData->Update(Filename);

    return OWOSensation;
}

UImportedAuth* UImportAuthFactory::CreateOWOFile(FString Filename, UObject* InParent, FName InName, EObjectFlags Flags)
{
    FString result;

    FFileHelper::LoadFileToString(result, *Filename);

    return NewObject<UImportedAuth>(InParent, UImportedAuth::StaticClass(), InName, Flags);
}