#include "UDPNetwork.h"

#define BUFLEN 1024 * 10
#define CONNECT_PORT 54020


using namespace OWOGame;

char* ToChars(std::string& value) { return const_cast<char*>(value.data()); }


TSharedPtr<FInternetAddr> GetAddr(std::string ip)
{
	FIPv4Address ipv4Adress;
	FIPv4Address::Parse(ip.c_str(), ipv4Adress);

	TSharedPtr<FInternetAddr> addr = ISocketSubsystem::Get(PLATFORM_SOCKETSUBSYSTEM)->CreateInternetAddr();

	addr->SetIp(ipv4Adress.Value);
	addr->SetPort(CONNECT_PORT);

	return addr;
}

void UDPNetwork::Initialize()
{
	if (socket == nullptr) {
		socket = FUdpSocketBuilder("udp socket")
			.AsNonBlocking()
			.AsReusable()
			.WithBroadcast();
	}
}

ConnectionState OWOGame::UDPNetwork::GetState()
{
	return state;
}

void OWOGame::UDPNetwork::SetState(ConnectionState newState)
{
	state = newState;
}

owoVector<owoString> OWOGame::UDPNetwork::GetConnectedAddressee()
{
	return connectedAddressee;
}

void UDPNetwork::Connect(std::string serverIp)
{
	bool isAlreadyConnected = false;

	for (int i = 0; i < connectedAddressee.size(); i++) {
		if (connectedAddressee[i] == serverIp)
			isAlreadyConnected = true;
	}
	if (!isAlreadyConnected)
		connectedAddressee.push_back(serverIp);
	state = ConnectionState::Connected;
}


std::string UDPNetwork::Listen(std::string& senderIp)
{
	TSharedPtr<FInternetAddr> address = ISocketSubsystem::Get(PLATFORM_SOCKETSUBSYSTEM)->CreateInternetAddr();
	std::string buf;

	int32 Read = 0;
	uint32 pending;

	uint8* bytesReceived = new uint8();

	while (socket && socket->HasPendingData(pending)) {
		socket->RecvFrom(bytesReceived, BUFLEN, Read, *address);
	}

	size_t bytesSize = sizeof(bytesReceived); 
	if (Read > 0 && Read < BUFLEN)
	{
		FString Ip;
		Ip = address->ToString(false);
		std::string ip = std::string(TCHAR_TO_UTF8(*Ip));
		senderIp = (std::string)TCHAR_TO_UTF8(*address->ToString(false));
		FString tempMessage = TCHAR_TO_UTF8(*CustomByteParser::BytesToStringCustom(bytesReceived, Read));

		if (tempMessage != "ping") {
			buf = (std::string)TCHAR_TO_UTF8(*tempMessage);
		}
			
	}

	return buf;
}

void UDPNetwork::Send(std::string message)
{
	SendTo(message, connectedAddressee[0]);
}

void UDPNetwork::SendTo(std::string message, std::string ip)
{
	TSharedPtr<FInternetAddr> addr = GetAddr(ip);
	addr->SetPort(CONNECT_PORT);

	FString messageF = message.c_str();
	TCHAR* serializedChar = messageF.GetCharArray().GetData();
	int32 size = FCString::Strlen(serializedChar);
	int32 sent = 0;

	if(socket)
		socket->SendTo((uint8*)TCHAR_TO_UTF8(serializedChar), size, sent, *addr);
}

void UDPNetwork::Disconnect()
{
	state = ConnectionState::Disconnected;
	if (socket != nullptr) {
		socket->Close();
		socket = nullptr;
	}
}
