// Copyright (C) 2023 OWO Games - All Rights Reserved
#pragma once

#include "CoreMinimal.h"
#include "Kismet/BlueprintFunctionLibrary.h"
#include "OWOUnrealUtilities.h"
THIRD_PARTY_INCLUDES_START
#include "../OWOAPI/Domain/Sensation.h"
#include "../OWOAPI/Domain/Muscle.h"
#include "../OWOAPI/Domain/String.h"
#include "../OWOAPI/Domain/MusclesGroup.h"
#include "../OWOAPI/Domain/SensationsFactory.h"
#include "../OWOAPI/Domain/SensationsSequence.h"
#include "../OWOAPI/Domain/SensationsParser.h"
#include "../OWOAPI/Domain/SensationWithMusclesParser.h"
#include "../OWOAPI/Domain/MusclesParser.h"
THIRD_PARTY_INCLUDES_END
#include "OWOLibrary.generated.h"


using namespace OWOGame;

/**
 * 
 */
UCLASS()
class OWOINTEGRATION_API UOWOLibrary : public UBlueprintFunctionLibrary
{
	GENERATED_BODY()
private:
	static sharedPtr<Sensation> ParseSensation(owoString sensation);
	static owoString AddSensationGroup(owoString muscles, sharedPtr<Sensation> sensation);
	static Muscle GetMuscle(owoVector<owoString> parameters);
	static FString ParseBakedSensation(owoString sensationString, owoString name, int id);
public:
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation", meta = (Tooltip = "Converts an imported .owo file to an usable sensation"))
	static FString ImportSensation(UImportedSensation* sensation);

	UFUNCTION(BlueprintPure, Category = "OWO|Sensation", meta = (Tooltip = "Converts an imported .owo file to an usable gameauth"))
	static FString ImportGameAuth(FString gameId, UImportedAuth* sensation);

	UFUNCTION(BlueprintPure, Category = "OWO|Sensation", meta = (Tooltip = "Bakes a sensation so that it can be used in a Game Auth"))
	static FString BakeSensation(FString sensation, FString name, int id);

	UFUNCTION(BlueprintPure, Category = "OWO|Sensation", meta = (Tooltip = "Changes the intensity of a sensation, the sensation must have muscles"))
		static FString ChangeSensationIntensity(FString sensation, int intensity);

	UFUNCTION(BlueprintPure, Category = "OWO|Sensation", meta = (Tooltip = "Builds a microsensation with the given parameters"))
	static FString BuildSensation(FString name = "", int Frequency = 0, float DurationInSeconds = 0.0f, int IntensityPercentage = 0, float RampUp = 0.0f, float RampDown = 0.0f, float ExitDelay = 0.0f, FString muscle = "");

	UFUNCTION(BlueprintPure, Category = "OWO|Sensation", meta = (Tooltip = "Appends two or more sensations to create a sensation chain", CommutativeAssociativeBinaryOperator = "true"))
	static FString BuildSensationChain(FString A, FString B);

	UFUNCTION(BlueprintPure, Category = "OWO|Muscle", meta = (Tooltip = "Builds a muscle to be added to a sensation"))
	static FString BuildMuscle(EOWOMusclesEnum Muscle, int IntensityPercentage = 100);

	UFUNCTION(BlueprintPure, Category = "OWO|Muscle", meta = (Tooltip = "Builds a muscle group", CommutativeAssociativeBinaryOperator = "true"))
	static FString BuildMuscleGroup(FString A, FString B);

	UFUNCTION(BlueprintPure, Category = "OWO|Muscle", meta = (Tooltip = "Builds a muscle group"))
	static FString BuildMuscleGroupFromArray(TArray<FString> muscles);
	
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation", meta = (Tooltip = "Adds muscles to a sensation"))
	static FString BuildSensationWithMuscle(FString Sensation, FString Muscle);

public:

	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Ball();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Dart();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Punch();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Grip();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString FreeFall();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString InsectBites();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString ShootEntry();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString ShootExit();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Bleeding();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString MachineGunRecoil();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString OneHandRecoil();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Dagger();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Shoot();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Heartbeat();
	UFUNCTION(BlueprintPure, Category = "OWO|Sensation|Predefined")
	static FString Hug();

public:

	UFUNCTION(BlueprintPure, Category = "OWO|Muscle|Predefined")
	static FString AllMuscles();
	UFUNCTION(BlueprintPure, Category = "OWO|Muscle|Predefined")
	static FString FrontMuscles();
	UFUNCTION(BlueprintPure, Category = "OWO|Muscle|Predefined")
	static FString FrontMusclesWithoutArms();
	UFUNCTION(BlueprintPure, Category = "OWO|Muscle|Predefined")
	static FString Pectorals();
	UFUNCTION(BlueprintPure, Category = "OWO|Muscle|Predefined")
	static FString Abdominals();
	UFUNCTION(BlueprintPure, Category = "OWO|Muscle|Predefined")
	static FString Arms();
	UFUNCTION(BlueprintPure, Category = "OWO|Muscle|Predefined")
	static FString BackMuscles();
};
