// Copyright (C) 2023 OWO Games - All Rights Reserved

#pragma once

#include "CoreMinimal.h"
#include "UObject/NoExportTypes.h"
#include "OWOUnrealUtilities.h"
#include "OWOLibrary.h"
THIRD_PARTY_INCLUDES_START
#include "../OWOAPI/Controller/OWO.h"
#include "../OWOAPI/Infraestructure/UDPNetwork.h"
#include "../OWOAPI/Domain/SensationsSequence.h"
#include "../OWOAPI/Domain/BakedSensationsParser.h"
THIRD_PARTY_INCLUDES_END
#include "UOWOController.generated.h"

using namespace OWOGame;


UCLASS(ClassGroup = (Custom), BlueprintType, Blueprintable, meta = (BlueprintSpawnableComponent))
class UOWOController : public UBlueprintFunctionLibrary
{
    GENERATED_BODY()
private:
    static uniquePtr<OWO> owo;
    static void Configure(std::vector<std::string> parameters);

    static void Initialize();
public:

    virtual void BeginDestroy() override;

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortToolTip = "Connects to server", ToolTip = "Tries to connect to a server with a known ip"))
        static void Connect(const FString& InAddress);

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortToolTip = "Updates the connection status"))
        static EOWOConnectionState UpdateStatus(float timeInSeconds);

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortToolTip = "Connects to a list of servers", ToolTip = "Tries to connect to a list server with known ips"))
        static void MultiConnect(TArray<FString> ips);

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortToolTip = "Creates Gameauth", ToolTip = "Creates a GameAuth with an id and a set of Baked Sensations"))
        static FString CreateGameAuth(FString gameId, TArray<FString> bakedSensations);

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortToolTip = "Configures the authentication", ToolTip = "Configures the authentication message with a GameAuth"))
        static void Configure(FString gameAuth);

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortTooltip = "Autoconnect to Socket", ToolTip = "Tries to connect to a socket sending a broadcast message and read a response from the app"))
        static void Autoconnect();

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortTooltip = "Scans for servers", ToolTip = "Scans the local network for listening OWO apps"))
        static void Scan(float timeInSeconds);

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortTooltip = "Returns avaliable Servers", ToolTip = "Returns a list with avaliable servers gotten from scan"))
        static TArray<FString> GetAvailableServers();

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortTooltip = "Returns Connection State", ToolTip = "Returns the current connection state of the socket"))
        static EOWOConnectionState ConnectionStatus();

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortToolTip = "Closes the socket", ToolTip = "Closes the socket and disconnects from the server"))
        static void Disconnect();

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortToolTip = "Send a sensantion", ToolTip = "Sends a sensation to the user"))
        static void SendSensation(FString sensation, FString muscle = "", int priority = 0);

    UFUNCTION(BlueprintCallable, Category = "OWO", meta = (ShortTooltip = "Stops ongoing sensation", ToolTip = "Stops the ongoing sensation if any"))
        static void StopCurrentSensation();


};

